package monitor;

/**
 *  Classe che implementa una variabile condizione.
 *  @author Cesare Stefanelli e Antonio Corradi
 *  @author modificato da G. C. e L.L.
 *  @version Monitor 1.1
 */
public class Cond {
	Monitor mon;
	int condCount;

         /**
	 * Costruttore della classe Cond;
	 *
	 * tipicamente una variabile condition viene istanziata in da un oggetto monitor in questo modo:
	 * <code>Cond c = new Cond(this);<code>
	 *
	 * @param mon il monitor in cui la variabile condizione
	 * &egrave; definita.
	 */

        public Cond(Monitor mon){
		this.mon = mon;
	}

	/** 
	 * Wait sulla variabile condizione, prima della sospensione rilascia il Mutex sul Monitor. 
	 */
	synchronized public void Wait() {
		condCount ++;
                if (mon.urgentCount > 0)
                {
                  // System.out.println(Thread.currentThread()+"processi in urgent: ne sveglio uno");
                  mon.urgentQueue.Signal();
                }
                else
                {
                  // System.out.println(Thread.currentThread()+"NO processi in urgent: signal su mutex");
                  mon.mutex.Signal();
                }
		try{ 
			wait(); 
		} catch(InterruptedException e){}
                Thread.currentThread().yield();
		condCount --;
	}

        synchronized private boolean SignalNascosta() {
		if (condCount > 0) {
			// System.out.println(Thread.currentThread()+"processi in condCount: ne sveglio uno");
			mon.urgentCount++;
			notify(); 
			return true;
		} else 
			return false;

	}

	/** 
	 * Signal sulla variabile condizione: risveglia un processo sospeso 
	 * e sospende il segnalante sulla Coda Urgent.
	 */
        public void Signal() {
		// System.out.println(Thread.currentThread()+"chiamo la signal nascosta");
		if (SignalNascosta())
		{
		  // System.out.println(Thread.currentThread()+"ho risvegliato: mi sospendo su urgent");
		  mon.urgentQueue.Wait();
		  // System.out.println(Thread.currentThread()+"sono stato risvegliato dalla urgent");
		}
	}

	/**
	 * Verifica la presenza di processi sospesi sulla Variabile Condizione.
	 * @return ritorna true se ci sono processi sospesi, false altrimenti
	 */
	synchronized public boolean queue() {
		if (condCount > 0) return true;
		else return false;
	}

}
